/****************************************************************************\
  FILE..........: tl30p_vc.h
  COPYRIGHT.....: Copyright(c) 1995, Bennet-Tec Information Systems
  VERSION.......: 3.99
  DESCRIPTION...: CTList class implementation
\****************************************************************************/

#include "stdafx.h"
#include "tl30p_vc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CTList, CVBControl)

/////////////////////////////////////////////////////////////////////////////

class CTListLoader
{
  private:
            HMODULE           m_hmod;
            FARPROC           m_fpFreeBuffer;
            FARPROC           m_fpIsValidBuffer;
            FARPROC           m_fpTranslateIndex;
            FARPROC           m_fpGetItemByXY;
            FARPROC           m_fpGetItemRect;
            FARPROC           m_fpSaveBuffer;
            FARPROC           m_fpLoadBuffer;
            FARPROC           m_fpCopyBuffer;
            FARPROC           m_fpPasteBuffer;
            FARPROC           m_fpIsClipboardFormatAvailable;
            FARPROC           m_fpIndexByBM;
            FARPROC           m_fpIsValidBM;
            FARPROC           m_fpFindItem;

  protected:
  public:
                              CTListLoader      ();
                             ~CTListLoader      ();

            void              FreeBuffer        (HTREEBUFFER);
            BOOL              IsValidBuffer     (HTREEBUFFER);
            int               TranslateIndex    (CTList*, int nFromIndexMethod,
                                                 int nToIndexMethod,
                                                 int nFromIndex);
            int               GetItemByXY       (CTList*, int nX, int nY, int nType);
            int               GetItemRect       (CTList*, int nIndex, int nOpts,
                                                 LPRECT lpRectT);
            int               SaveBuffer        (HTREEBUFFER hTreeBuffer,
                                                 int nFile);
            int               LoadBuffer        (HTREEBUFFER hTreeBuffer,
                                                 int nFile);
            int               CopyBuffer        (HTREEBUFFER hTreeBuffer);
            int               PasteBuffer       (HTREEBUFFER hTreeBuffer);
            int               IsClipboardFormatAvailable
                                                ();
            int               IndexByBM         (CTList*, LONG hTreeBookmark);
            int               IsValidBM         (CTList*, LONG hTreeBookmark);
            int               FindItem          (CTList*, LPSTR lpFindWhat, int nFlags,
                                                 int nFromIndex, int nToIndex);
};


CTListLoader g_TListLoader;

/////////////////////////////////////////////////////////////////////////////
// CTList

void CTList::FreeBuffer(HTREEBUFFER hTreeBuffer)
{
  g_TListLoader.FreeBuffer(hTreeBuffer);
}

BOOL CTList::IsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.IsValidBuffer(hTreeBuffer);
}

int CTList::TranslateIndex
(
  int nFromIndexMethod, int nToIndexMethod, int nFromIndex
)
{
  return g_TListLoader.TranslateIndex(
                         this, nFromIndexMethod, nToIndexMethod, nFromIndex);
}

int CTList::GetItemByXY(int nX, int nY, int nType)
{
  return g_TListLoader.GetItemByXY(this, nX, nY, nType);
}

int CTList::GetItemRect(int nIndex, int nOpts, LPRECT lpRectT)
{
  return g_TListLoader.GetItemRect(this, nIndex, nOpts, lpRectT);
}

int CTList::SaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.SaveBuffer(hTreeBuffer, nFile);
}

int CTList::LoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.LoadBuffer(hTreeBuffer, nFile);
}

int CTList::CopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.CopyBuffer(hTreeBuffer);
}

int CTList::PasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.PasteBuffer(hTreeBuffer);
}

int CTList::IsClipboardFormatAvailable()
{
  return g_TListLoader.IsClipboardFormatAvailable();
}

int CTList::IndexByBM(LONG hTreeBookmark)
{
  return g_TListLoader.IndexByBM(this, hTreeBookmark);
}

int CTList::IsValidBM(LONG hTreeBookmark)
{
  return g_TListLoader.IsValidBM(this, hTreeBookmark);
}

int CTList::FindItem(LPSTR lpFindWhat, int nFlags, int nFromIndex, int nToIndex)
{
  return g_TListLoader.FindItem(
                         this, lpFindWhat, nFlags, nFromIndex, nToIndex);
}


BEGIN_MESSAGE_MAP(CTList, CVBControl)
  //{{AFX_MSG_MAP(CTList)
    // NOTE - the ClassWizard will add and remove mapping macros here.
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

#ifdef _DEBUG

void CTList::AssertValid() const
{
  CVBControl::AssertValid();
}

void CTList::Dump(CDumpContext& dc) const
{
  CVBControl::Dump(dc);
}

#endif

/////////////////////////////////////////////////////////////////////////////
// CTList message handlers

CTListLoader::CTListLoader()
{
  m_hmod = LoadLibrary("TLISTPRO.VBX");

  if(m_hmod >= HINSTANCE_ERROR)
  {
    m_fpFreeBuffer     = GetProcAddress(m_hmod, "TListFreeBuffer");
    m_fpIsValidBuffer  = GetProcAddress(m_hmod, "TListIsValidBuffer");
    m_fpTranslateIndex = GetProcAddress(m_hmod, "TListTranslateIndex");
    m_fpGetItemByXY    = GetProcAddress(m_hmod, "TListGetItemByXY");
    m_fpGetItemRect    = GetProcAddress(m_hmod, "TListGetItemRect");
    m_fpSaveBuffer     = GetProcAddress(m_hmod, "TListSaveBuffer");
    m_fpLoadBuffer     = GetProcAddress(m_hmod, "TListLoadBuffer");
    m_fpCopyBuffer     = GetProcAddress(m_hmod, "TListCopyBuffer");
    m_fpPasteBuffer    = GetProcAddress(m_hmod, "TListPasteBuffer");
    m_fpIsClipboardFormatAvailable =
                         GetProcAddress(m_hmod,
                           "TListIsClipboardFormatAvailable");
    m_fpIndexByBM      = GetProcAddress(m_hmod, "TListIndexByBM");
    m_fpIsValidBM      = GetProcAddress(m_hmod, "TListIsValidBM");
    m_fpFindItem       = GetProcAddress(m_hmod, "TListFindItem");
  }
  else
  {
    m_fpFreeBuffer    = NULL;
    m_fpIsValidBuffer = NULL;
    m_hmod = NULL;
    TRACE0("Couldn't load TLISTPRO.VBX\n");
  }
}

CTListLoader::~CTListLoader()
{
  if(m_hmod)
    FreeLibrary(m_hmod);
}

void CTListLoader::FreeBuffer(HTREEBUFFER hTreeBuffer)
{
  if(m_fpFreeBuffer)
    (*(void (WINAPI *)(HTREEBUFFER))m_fpFreeBuffer)(hTreeBuffer);
}

BOOL CTListLoader::IsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpIsValidBuffer
         ? (*(BOOL (WINAPI *)(HTREEBUFFER))m_fpIsValidBuffer)(hTreeBuffer)
         : FALSE;
}

int CTListLoader::TranslateIndex
(
  CTList *pListTree, int nFromIndexMethod, int nToIndexMethod, int nFromIndex
)
{
  ASSERT(pListTree);
  return m_fpTranslateIndex
         ? (*(int (WINAPI *)(HCTL, int, int, int))
            m_fpTranslateIndex)(pListTree->GetHCTL(), nFromIndexMethod,
                                nToIndexMethod, nFromIndex)
         : FALSE;
}

int CTListLoader::GetItemByXY(CTList *pListTree, int nX, int nY, int nType)
{
  ASSERT(pListTree);
  return m_fpGetItemByXY
         ? (*(int (WINAPI *)(HCTL, int, int, int))
            m_fpGetItemByXY)(pListTree->GetHCTL(), nX, nY, nType)
         : FALSE;
}

int CTListLoader::GetItemRect(CTList *pListTree, int nIndex, int nOpts,
                              LPRECT lpRectT)
{
  ASSERT(pListTree);
  return m_fpGetItemRect
         ? (*(int (WINAPI *)(HCTL, int, int, LPRECT))
            m_fpGetItemRect)(pListTree->GetHCTL(), nIndex, nOpts, lpRectT)
         : FALSE;
}

int CTListLoader::SaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return m_fpSaveBuffer
         ? (*(int (WINAPI *)(HTREEBUFFER, int))m_fpSaveBuffer)(hTreeBuffer, nFile)
         : FALSE;
}

int CTListLoader::LoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return m_fpLoadBuffer
         ? (*(int (WINAPI *)(HTREEBUFFER, int))m_fpLoadBuffer)(hTreeBuffer, nFile)
         : FALSE;
}

int CTListLoader::CopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpCopyBuffer
         ? (*(int (WINAPI *)(HTREEBUFFER))m_fpCopyBuffer)(hTreeBuffer)
         : FALSE;
}

int CTListLoader::PasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpPasteBuffer
         ? (*(int (WINAPI *)(HTREEBUFFER))m_fpPasteBuffer)(hTreeBuffer)
         : FALSE;
}

int CTListLoader::IsClipboardFormatAvailable()
{
  return m_fpIsClipboardFormatAvailable
         ? (*(int (WINAPI *)())m_fpIsClipboardFormatAvailable)()
         : FALSE;
}

int CTListLoader::IndexByBM(CTList *pListTree, LONG hTreeBookmark)
{
  ASSERT(pListTree);
  return m_fpIndexByBM
         ? (*(int (WINAPI *)(HCTL, LONG))
           m_fpIndexByBM)(pListTree->GetHCTL(), hTreeBookmark)
         : FALSE;
}

int CTListLoader::IsValidBM(CTList *pListTree, LONG hTreeBookmark)
{
  ASSERT(pListTree);
  return m_fpIsValidBM
         ? (*(int (WINAPI *)(HCTL, LONG))
            m_fpIsValidBM)(pListTree->GetHCTL(), hTreeBookmark)
         : FALSE;
}

int CTListLoader::FindItem(CTList *pListTree, LPSTR lpFindWhat,
                           int nFlags, int nFromIndex, int nToIndex)
{
  ASSERT(pListTree);
  return m_fpFindItem
         ? (*(int (WINAPI *)(HCTL, LPSTR, int, int, int))
            m_fpFindItem)(pListTree->GetHCTL(), lpFindWhat, nFlags,
                          nFromIndex, nToIndex)
         : FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// TList's functions

VOID WINAPI TListFreeBuffer(HTREEBUFFER hTreeBuffer)
{
  g_TListLoader.FreeBuffer(hTreeBuffer);
}

int  WINAPI TListIsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.IsValidBuffer(hTreeBuffer);
}

int WINAPI TListTranslateIndex(CTList* pListTree, int nFromIndexMethod,
                               int nToIndexMethod, int nFromIndex)
{
  return g_TListLoader.TranslateIndex(pListTree, nFromIndexMethod,
                                      nToIndexMethod, nFromIndex);
}

int WINAPI TListGetItemByXY(CTList* pListTree, int nX, int nY, int nType)
{
  return g_TListLoader.GetItemByXY(pListTree, nX, nY, nType);
}

int WINAPI TListGetItemRect(CTList* pListTree, int nIndex, int nOpts,
                            LPRECT lpRectT)
{
  return g_TListLoader.GetItemRect(pListTree, nIndex, nOpts, lpRectT);
}

int WINAPI TListSaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.SaveBuffer(hTreeBuffer, nFile);
}

int WINAPI TListLoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.LoadBuffer(hTreeBuffer, nFile);
}

int WINAPI TListCopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.CopyBuffer(hTreeBuffer);
}

int WINAPI TListPasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.PasteBuffer(hTreeBuffer);
}

int WINAPI TListIsClipboardFormatAvailable()
{
  return g_TListLoader.IsClipboardFormatAvailable();
}

int WINAPI TListIndexByBM(CTList* pListTree, LONG hTreeBookmark)
{
  return g_TListLoader.IndexByBM(pListTree, hTreeBookmark);
}

int WINAPI TListIsValidBM(CTList* pListTree, LONG hTreeBookmark)
{
  return g_TListLoader.IsValidBM(pListTree, hTreeBookmark);
}

int WINAPI TListFindItem(CTList* pListTree, LPSTR lpFindWhat,
                         int nFlags, int nFromIndex, int nToIndex)
{
  return g_TListLoader.FindItem(pListTree, lpFindWhat, nFlags,
                                nFromIndex, nToIndex);
}
